package eswtshowcase.modules;

import java.util.Random;
import org.eclipse.ercp.swt.mobile.CaptionedControl;
import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Slider;

import org.eclipse.swt.custom.ScrolledComposite;

/**
 * A demonstration for the Button and Slider components along with the
 * Canvas drawing capabilities
 */
public class ButtonSliderCanvasDemoModule implements DemoModule, SelectionListener {
    private Shell demoShell;
    private ScrolledComposite scrolledComposite;
    private Composite demoComposite;   
    private Command backCommand;
    private Command drawCommand;
    
    private Button rectButton;
    private Button lineButton;
    private Button circleButton;
    private Button polygonButton;
    
    private Button threeVerticesButton;
    private Button fiveVerticesButton;
    private Button sevenVerticesButton;
    
    private Slider colorSlider;
        
    public String getTitle(){
        return "Button, Slider & Canvas";
    }
    
    public String getDescription(){
        return "Demonstrates the Button, Slider and Canvas (GC) components";
    }
    
    public void runDemo(Shell parent){
        demoShell = new Shell(SWT.MAX);
        demoShell.setText("Buttons & Sliders");
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));
        
        scrolledComposite = new ScrolledComposite(demoShell, SWT.V_SCROLL);
        demoComposite = new Composite(scrolledComposite, SWT.NONE);
        demoComposite.setLayout(new GridLayout(1, true));
        scrolledComposite.setContent(demoComposite);
        
        backCommand = new Command(demoShell, Command.EXIT, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);
        
        drawCommand = new Command(demoShell, Command.OK, 0);
        drawCommand.setText("Draw!");
        drawCommand.addSelectionListener(this);
        
        CaptionedControl shapeCaption = new CaptionedControl(demoComposite, SWT.NONE);
        shapeCaption.setText("Draw Shapes:");
        
        Composite shapeComposite = new Composite(shapeCaption, SWT.NONE);
        GridLayout layout = new GridLayout(2, true);
        shapeComposite.setLayout(layout);
        
        //create check buttons for the shape selections 
        rectButton = new Button(shapeComposite, SWT.CHECK);
        rectButton.setText("Rect");
        rectButton.addSelectionListener(this);
        
        lineButton = new Button(shapeComposite, SWT.CHECK);
        lineButton.setText("Line");
        lineButton.addSelectionListener(this);
        
        circleButton = new Button(shapeComposite, SWT.CHECK);
        circleButton.setText("Circle");
        circleButton.addSelectionListener(this);
        
        polygonButton = new Button(shapeComposite, SWT.CHECK);
        polygonButton.setText("Polygon");
        polygonButton.addSelectionListener(this);
        
        CaptionedControl verticesCaption = new CaptionedControl(demoComposite, SWT.NONE);
        verticesCaption.setText("Polygon Vertices:");
        
        Composite verticesComposite = new Composite(verticesCaption, SWT.NONE);
        verticesComposite.setLayout(new FillLayout(SWT.HORIZONTAL));
        
        //create radio buttons for the vertice amount selection,
        //the radio functionality (only one of them can be selected at once)
        //is handled automatically
        threeVerticesButton = new Button(verticesComposite, SWT.RADIO);
        threeVerticesButton.setText("3");
        threeVerticesButton.addSelectionListener(this);
        
        fiveVerticesButton = new Button(verticesComposite, SWT.RADIO);
        fiveVerticesButton.setText("5");
        fiveVerticesButton.addSelectionListener(this);
        
        sevenVerticesButton = new Button(verticesComposite, SWT.RADIO);
        sevenVerticesButton.setText("7");
        sevenVerticesButton.addSelectionListener(this);
        
        //the initial selection
        threeVerticesButton.setSelection(true); 
        
        CaptionedControl colorCaption = new CaptionedControl(demoComposite, SWT.NONE);
        colorCaption.setText("Color Level:");
        
        //create a horizontal slider for the color level
        colorSlider = new Slider(colorCaption, SWT.HORIZONTAL);
        //setValues(int selection, int minimum, int maximum, 
        //          int thumb, int increment, int pageIncrement)
        colorSlider.setValues(100, 0, 100, 5, 5, 10);
        colorSlider.addSelectionListener(this);
        
        demoComposite.setSize(demoComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
        demoComposite.layout();
        //demoShell.layout();
        demoShell.open();
        demoShell.setActive();
    }
    
    public void widgetSelected(SelectionEvent e){
        if(e.widget.equals(backCommand)){
            back();
        } else if(e.widget.equals(drawCommand)){
            draw();
        }
    }
    
    public void widgetDefaultSelected(SelectionEvent e){
        
    }
    
    private void back(){
        demoShell.dispose();
        demoShell = null;
    }
    
    private void draw(){
        //create a painter object and initialize it with the parameters
        //the user has selected
        Painter painter = new Painter(colorSlider.getSelection());
        painter.setPaintStatuses(rectButton.getSelection(), 
                lineButton.getSelection(), 
                circleButton.getSelection(),
                polygonButton.getSelection());
        
        int vertices = 0;
        if(threeVerticesButton.getSelection()){
            vertices = 3;
        } else if(fiveVerticesButton.getSelection()){
            vertices = 5;
        } else if(sevenVerticesButton.getSelection()){
            vertices = 7;
        }
        painter.setPolygonVertices(vertices);
       
        //start painting
        painter.start();
    }
    
    private static class Painter implements PaintListener, SelectionListener {
        Random rand;
        
        int colorLevel;
        boolean paintRect = false;
        boolean paintLine = false;
        boolean paintCircle = false;
        boolean paintPolygon = false;
        int polygonVertices;
        
        Shell shell;
        int width;
        int height;
        
        public Painter(int colorLevel){
            this.colorLevel = colorLevel;
            
            shell = new Shell(Display.getCurrent(), SWT.NO_BACKGROUND);
            shell.setText("Canvas Drawing");
            shell.setLayout(new FillLayout());
            shell.addPaintListener(this);
            shell.open();
            
            Command stopCommand = new Command(shell, Command.EXIT, 0);
            stopCommand.setText("Stop!");
            stopCommand.addSelectionListener(this);
            
            width = shell.getSize().x;
            height = shell.getSize().y;
            
            rand = new Random();
        }
        
        //this method is called on every redraw
        public void paintControl(PaintEvent e){
            //choose a random color and set the bg and fg colors
            Color col = getColor();
            e.gc.setForeground(col);
            e.gc.setBackground(col);
            
            if(paintRect){
                //paint a filled rectangle in a random location
                e.gc.fillRectangle(rand.nextInt(width), rand.nextInt(height),
                        rand.nextInt(50), rand.nextInt(50));          
            }
            if(paintLine){
                //paint a line in a random location
                e.gc.drawLine(rand.nextInt(width), rand.nextInt(height), 
                        rand.nextInt(width), rand.nextInt(height));
            }
            if(paintCircle){
                //paint a filled circle in a random location
                int radius = rand.nextInt(50);
                e.gc.fillOval(rand.nextInt(width), rand.nextInt(height), 
                        radius, radius);
            }
            if(paintPolygon){
                //paint a polygon in a random location
                e.gc.drawPolygon(getVertices(polygonVertices));
            }
            
            //colors are graphical items that have to be explicitly disposed
            //after use
            col.dispose();
        }
        
        public void widgetSelected(SelectionEvent e){
            //we only have one command
            shell.dispose();
        }
        
        public void widgetDefaultSelected(SelectionEvent e){
            
        }
        
        public void start(){
            //our event loop, we have to have this to be able to call redraw
            //consistently
            while(!shell.isDisposed()){
                shell.redraw();
                
                Display.getCurrent().readAndDispatch();
                
                //we want to keep a somewhat consistent frame rate even though
                //there would be more events to handle
                try {
                    Thread.sleep(100);
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }
            }
        }
        
        //helper method that gets a random colour with the grayscale filter applied
        private Color getColor(){
            int r = rand.nextInt(256);
            int g = rand.nextInt(256);
            int b = rand.nextInt(256);
            
            //get the grayscale intensity, the multipliers are related to how
            //the human eye percieves the intensity of the different components
            int gray = (r*21 + g*72 + b*7)/100;
            
            //combine the colours with the grayscale versions,
            //weigh them using colorLevel
            int grayWeighted = gray * (100-colorLevel)/100;
            r = r * colorLevel/100 + grayWeighted;
            g = g * colorLevel/100 + grayWeighted;
            b = b * colorLevel/100 + grayWeighted;
            
            return new Color(Display.getCurrent(), r, g, b);
        }
        
        //helper method that creates vertices in random locations
        private int[] getVertices(int verticeAmount){
            //we have to create an int array double the size because the
            //x and y coordinates alternate
            int[] vertices = new int[verticeAmount*2];
            for(int i=0; i < vertices.length; i=i+2){
                vertices[i] = rand.nextInt(width); //x
                vertices[i+1] = rand.nextInt(height); //y
            }
            return vertices;
        }
        
        public void setPaintStatuses(boolean rect, boolean line, boolean circle, boolean polygon){
            this.paintRect = rect;
            this.paintLine = line;
            this.paintCircle = circle;
            this.paintPolygon = polygon;
        }
        
        public void setPolygonVertices(int vertices){
            this.polygonVertices = vertices;
        }
    }
}
