package eswtshowcase.modules;

import org.eclipse.ercp.swt.mobile.CaptionedControl;
import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.ercp.swt.mobile.TimedMessageBox;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
//import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import org.eclipse.swt.custom.ScrolledComposite;
/**
 * A demonstration of the Combo, MessageBox and TimedMessageBox components.
 */
public class ComboMessageBoxDemoModule implements DemoModule, SelectionListener {
    private Shell demoShell;
    private Composite demoComposite;
    private ScrolledComposite scrolledComposite;    
    private Command backCommand;
    private Command displayBoxCommand;
    
    private Combo typeCombo;
    private Combo iconCombo;
    private Combo textCombo;
    private Combo commandsCombo;
    
    private CaptionedControl commandsCaption;
    
    public String getTitle(){
        return "Combo & MessageBox";
    }
    
    public String getDescription(){
        return "Demonstrates the Combo and MessageBox components";
    }
    
    public void runDemo(Shell parent){
        demoShell = new Shell(SWT.MAX);
        demoShell.setText("Combos and MessageBoxes");
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));
             
        scrolledComposite = new ScrolledComposite(demoShell, SWT.V_SCROLL);
        demoComposite = new Composite(scrolledComposite, SWT.NONE);
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));
        scrolledComposite.setContent(demoComposite);
        
        backCommand = new Command(demoShell, Command.EXIT, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);
        
        displayBoxCommand = new Command(demoShell, Command.OK, 0);
        displayBoxCommand.setText("Display");
        displayBoxCommand.addSelectionListener(this);
        
        //Create a combo box for the message box type
        CaptionedControl typeCaption = new CaptionedControl(demoComposite, SWT.NONE);
        typeCaption.setText("Message Box Type");
        typeCombo = new Combo(typeCaption, SWT.DROP_DOWN|SWT.READ_ONLY);
        typeCombo.setItems(new String[] {"MessageBox", "TimedMessageBox"});
        typeCombo.addSelectionListener(this);
        typeCombo.select(0); //select MessageBox initially
        
        //create a combo box for the icon
        CaptionedControl iconCaption = new CaptionedControl(demoComposite, SWT.NONE);
        iconCaption.setText("Icon");
        iconCombo = new Combo(iconCaption, SWT.DROP_DOWN|SWT.READ_ONLY);
        iconCombo.setItems(new String[] {"Error", "Information", "Question", "Warning", "Working" });
        iconCombo.addSelectionListener(this);
        iconCombo.select(0);
        
        //create a combo box for the text
        CaptionedControl textCaption = new CaptionedControl(demoComposite, SWT.NONE);
        textCaption.setText("Text");
        textCombo = new Combo(textCaption, SWT.DROP_DOWN);
        textCombo.setItems(new String[]{"This is a message box.", "Hooray!"});
        textCombo.addSelectionListener(this);
        textCombo.select(0);
        
        //create a combo box for the commands
        commandsCaption = new CaptionedControl(demoComposite, SWT.NONE);
        commandsCaption.setText("Commands");
        commandsCombo = new Combo(commandsCaption, SWT.DROP_DOWN|SWT.READ_ONLY);
        commandsCombo.setItems(new String[]{"OK", "OK & Cancel", "Yes & No", "Yes, No & Cancel", "Retry & Cancel"});
        commandsCombo.addSelectionListener(this);
        commandsCombo.select(0);
        
        demoComposite.setSize(demoComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
        demoComposite.layout();
        
        demoShell.open();
        demoShell.setActive();
    }

    public void widgetSelected(SelectionEvent e){
        if(e.widget.equals(backCommand)){
            back();
        } else if(e.widget.equals(displayBoxCommand)){
            displayBox();
        } else if(e.widget.equals(typeCombo)){
            //if the user selects MessageBox he can also choose the buttons
            //a TimedMessageBox doesn't have them so we disable the combo box
            if(typeCombo.getSelectionIndex() == 0){
                commandsCombo.setEnabled(true);
                commandsCaption.setVisible(true);
            } else {
                commandsCombo.setEnabled(false);
                commandsCaption.setVisible(false);
            }
        }
    }
    
    public void widgetDefaultSelected(SelectionEvent e){
        
    }
    
    private void back(){
        demoShell.dispose();
        demoShell = null;
    }
    
    private void displayBox(){
        int style = getStyle();
        
        //if it's a message box
        if(typeCombo.getSelectionIndex() == 0){
            //message boxes have buttons, add them to the style
            style |= getCommands();
            
            //create a MessageBox with the style
            MessageBox box = new MessageBox(demoShell, style);
            box.setMessage(textCombo.getText());
            int result = box.open();
            
            //here we could test the result type against the buttons the user chose
            //eg. if(result == SWT.YES)
        } else {
            //create a TimedMessageBox with the style
            TimedMessageBox box = new TimedMessageBox(demoShell, style);
            box.setMessage(textCombo.getText());
            box.open();
        }
    }
    
    //gets the box style (i.e. icon) depending on the user choice
    private int getStyle(){
        switch(iconCombo.getSelectionIndex()){
            case 0:
                return SWT.ICON_ERROR;
            case 1:
                return SWT.ICON_INFORMATION;
            case 2:
                return SWT.ICON_QUESTION;
            case 3:
                return SWT.ICON_WARNING;
            case 4:
                return SWT.ICON_WORKING;
            default:
                return SWT.ICON_ERROR;       
        }
    }
    
    //gets the command buttons the user has selected
    private int getCommands(){
        switch(commandsCombo.getSelectionIndex()){
            case 0:
                return SWT.OK;
            case 1:
                return SWT.OK|SWT.CANCEL;
            case 2:
                return SWT.YES|SWT.NO;
            case 3:
                return SWT.YES|SWT.NO|SWT.CANCEL;
            case 4:
                return SWT.RETRY|SWT.CANCEL;
            default:
                return SWT.OK;
        }
    }
}

