package eswtshowcase.modules;

import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.ercp.swt.mobile.QueryDialog;
import org.eclipse.ercp.swt.mobile.TimedMessageBox;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;

/**
 * Demonstrates the List component by using a multiple selection shopping list.
 */
public class ListDemoModule implements DemoModule, SelectionListener {
    private Shell demoShell;
    private Shell parent;
    private Composite demoComposite;
    private List demoList;
    
    private Command backCommand;
    private Command addCommand;
    private Command removeCommand;
    
    private static final String[] originalContent = new String[]{ "Apples", "Oranges", "Grapefruit", "Milk" };
    
    public ListDemoModule(){        
    }
    
    public void runDemo(Shell parent){
        this.parent = parent;
        demoShell = new Shell(SWT.MAX);
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));
        demoShell.setText("Shopping List");
        
        demoComposite = new Composite(demoShell, SWT.NONE);
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));
        demoShell.layout();
        
        backCommand = new Command(demoShell, Command.EXIT, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);
        
        addCommand = new Command(demoShell, Command.SELECT, 1);
        addCommand.setText("Add");
        addCommand.addSelectionListener(this);
        
        removeCommand = new Command(demoShell, Command.SELECT, 1);
        removeCommand.setText("Remove");
        removeCommand.addSelectionListener(this);
        
        //first we create a multiple selection list
        demoList = new List(demoComposite, SWT.MULTI);
        demoList.addSelectionListener(this);
        //then we set the original content
        demoList.setItems(originalContent);
        
        demoComposite.layout();
        demoShell.open();
        demoShell.setActive();
    }
    
    public String getTitle(){
        return "Shopping List";
    }
    
    public String getDescription(){
        return "Demonstrates the List component";
    }
    
    public void widgetSelected(SelectionEvent e){
        if(e.widget.equals(backCommand)){
            back();
        } else if(e.widget.equals(addCommand)){
            add();
        } else if(e.widget.equals(removeCommand)){
            remove();
        }
    }
    
    public void widgetDefaultSelected(SelectionEvent e){
    }
    
    private void back(){
        demoShell.dispose();
        demoShell = null;
        parent.getShell().setActive();
    }
    
    private void add(){
        //here we first create a dialog to ask the user for a new item
        QueryDialog dialog = new QueryDialog(demoShell, QueryDialog.STANDARD);
        dialog.setPromptText("I should also remember:", "");
        String itemToAdd = dialog.open();
        
        //check whether the user pressed the cancel button or is trying to add
        //a duplicate item
        if(itemToAdd == null){
            return;
        } else if(demoList.indexOf(itemToAdd) != -1){
            showMessage("One entry of " + itemToAdd + 
                    " should be enough for you to remember...");
            return;
        }
        
        //add the item to the list
        demoList.add(itemToAdd);
    }
    
    private void remove(){
        //if the user hasn't selected anything
        if(demoList.getSelectionCount() == 0){
            showMessage("Please mark the items you want to remove first!");
            return;
        }
        
        //remove the selections from the list
        demoList.remove(demoList.getSelectionIndices());
    }
    
    private void showMessage(String message){
        TimedMessageBox box = new TimedMessageBox(demoShell, SWT.ICON_ERROR);
        box.setMessage(message);
        box.open();
    }
}
