package eswtshowcase.modules;

import java.io.InputStream;
import org.eclipse.ercp.swt.mobile.CaptionedControl;
import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.ercp.swt.mobile.HyperLink;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

/** 
 * Demonstrates the static components (Label (also with an image), CaptionedControl and HyperLink)
 * along with the different layouts (FillLayout, FormLayout, GridLayout and RowLayout).
 * Feel free to modify the layout switching methods to try out things.
 */
public class StaticAndLayoutsDemoModule implements DemoModule, SelectionListener{
    private Shell demoShell;
    private Composite demoComposite;
    
    private Command backCommand;
    private Command fillLayoutCommand;
    private Command formLayoutCommand;
    private Command gridLayoutCommand;
    private Command rowLayoutCommand;
    
    private Image angryImage;
    
    private Label text;
    private CaptionedControl caption;
    private HyperLink link;
    private Label img;
    
    public String getTitle(){
        return "Static Components & Layouts";
    }
    
    public String getDescription(){
        return "Demonstrates the CaptionedControl, Label, Hyperlink and Image component and the layouts";
    }
    
    public void runDemo(Shell parent){
        demoShell = new Shell(SWT.MAX);
        demoShell.setText("Statics");
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));
        
        demoComposite = new Composite(demoShell, SWT.NONE);
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));
        demoShell.layout();
        
        backCommand = new Command(demoShell, Command.EXIT, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);
        
        fillLayoutCommand = new Command(demoShell, Command.SELECT, 1);
        fillLayoutCommand.setText("Fill Layout");
        fillLayoutCommand.addSelectionListener(this);
        
        formLayoutCommand = new Command(demoShell, Command.SELECT, 1);
        formLayoutCommand.setText("Form Layout");
        formLayoutCommand.addSelectionListener(this);
        
        gridLayoutCommand = new Command(demoShell, Command.SELECT, 1);
        gridLayoutCommand.setText("Grid Layout");
        gridLayoutCommand.addSelectionListener(this);
        
        rowLayoutCommand = new Command(demoShell, Command.SELECT, 1);
        rowLayoutCommand.setText("Row Layout");
        rowLayoutCommand.addSelectionListener(this);
        
        //Create a Label with some text
        text = new Label(demoComposite, SWT.LEFT);
        text.setText("Text");
        
        //Create a caption and insert a read only Text widget into it
        caption = new CaptionedControl(demoComposite, SWT.NONE);
        caption.setText("Caption");
        
        //Create a HyperLink that behaves like an e-mail address
        link = new HyperLink(demoComposite, SWT.NONE, HyperLink.EMAIL);
        link.setText("a@b.com");
                
        //Load up an image to use with the image label, we have to remember
        //to dispose it afterwards!
        InputStream is = getClass().getResourceAsStream("/angry.PNG");
        angryImage = new Image(demoShell.getDisplay(), is);
        
        //Create a Label for the image
        img = new Label(demoComposite, SWT.LEFT);
        img.setImage(angryImage);
        
        //start with FillLayout
        switchToFill();
        demoShell.open();
        demoShell.setActive();
    }
    
    public void widgetSelected(SelectionEvent e){
        if(e.widget.equals(backCommand)){
            back();
        } else if(e.widget.equals(fillLayoutCommand)){
            switchToFill();
        } else if(e.widget.equals(formLayoutCommand)){
            switchToForm();
        } else if(e.widget.equals(gridLayoutCommand)){
            switchToGrid();
        } else if(e.widget.equals(rowLayoutCommand)){
            switchToRow();
        }
    }
    
    public void widgetDefaultSelected(SelectionEvent e){
        
    }
    
    private void back(){
        demoShell.dispose();
        demoShell = null;
        
        angryImage.dispose();
    }
    
    private void switchToFill(){
        //FillLayouts are pretty simple, you just specify whether you want them
        //to fill up horizontally or vertically
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));
        
        fillLayoutCommand.setEnabled(false);
        formLayoutCommand.setEnabled(true);
        gridLayoutCommand.setEnabled(true);
        rowLayoutCommand.setEnabled(true);
        
        demoComposite.layout();
    }
    
    private void switchToForm(){
        //FormLayouts are the most customizable of the Layouts
        demoComposite.setLayout(new FormLayout());
        
        //by default the objects are in the upper left corner
        text.setLayoutData(new FormData());
        
        //here we set up the data for our caption, we want the caption to start
        //from the bottom right corner of the text box, extend all the way to
        //the right horizontally and to the center of the screen vertically
        //notice how we can create FormAttachments using the controls themselves
        FormData captionData = new FormData();
        captionData.top = new FormAttachment(text);
        captionData.bottom = new FormAttachment(50);
        captionData.left = new FormAttachment(text);
        captionData.right = new FormAttachment(100);
        caption.setLayoutData(captionData);
        
        //we want the hyperlink to show right below the caption box, insert
        //a small margin on the borders
        FormData linkData = new FormData();
        linkData.top = new FormAttachment(caption, 5);
        linkData.bottom = new FormAttachment(100, -5);
        linkData.left = new FormAttachment(text, 5);
        linkData.right = new FormAttachment(100, -5);
        link.setLayoutData(linkData);
        
        //the image label should show up to the left of the hyperlink
        FormData imgData = new FormData();
        imgData.top = new FormAttachment(caption);
        imgData.bottom = new FormAttachment(100);
        imgData.left = new FormAttachment(0);
        imgData.right = new FormAttachment(link);
        img.setLayoutData(imgData);
        
        fillLayoutCommand.setEnabled(true);
        formLayoutCommand.setEnabled(false);
        gridLayoutCommand.setEnabled(true);
        rowLayoutCommand.setEnabled(true);
        
        demoComposite.layout();
    }
    
    private void switchToGrid(){
        //GridLayouts are pretty simple to use, we just specify how many columns
        //we want to have and whether we want all of the columns to be of equal
        //width.
        demoComposite.setLayout(new GridLayout(2, false));
        
        //We have to make sure the layout datas are GridDatas, since if they 
        //aren't null (for example if we checked out the FormLayout beforehand
        //they have FormData attached to them) the old layout datas don't get
        //overridden
        text.setLayoutData(new GridData());
        caption.setLayoutData(new GridData());
        link.setLayoutData(new GridData());
        img.setLayoutData(new GridData());
        
        fillLayoutCommand.setEnabled(true);
        formLayoutCommand.setEnabled(true);
        gridLayoutCommand.setEnabled(false);
        rowLayoutCommand.setEnabled(true);
        
        demoComposite.layout();
    }
    
    private void switchToRow(){
        //In a RowLayout the component just get inserted in a neat row, this is
        // more useful in a bunch of small components such as radio buttons.
        demoComposite.setLayout(new RowLayout(SWT.HORIZONTAL));
        
        //Make sure the widgets have the correct layout data attached to them
        //see switchToGrid for more complete explanation.
        text.setLayoutData(new RowData());
        caption.setLayoutData(new RowData());
        link.setLayoutData(new RowData());
        img.setLayoutData(new RowData());
        
        fillLayoutCommand.setEnabled(true);
        formLayoutCommand.setEnabled(true);
        gridLayoutCommand.setEnabled(true);
        rowLayoutCommand.setEnabled(false);
        
        demoComposite.layout();
    }
}
