package eswtshowcase.modules;

import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

/**
 * Demonstrates the Tree widget with a inheritance hierarchy of the
 * org.eclipse.swt.widgets package.
 */
public class TreeDemoModule implements DemoModule, SelectionListener{
    private Shell demoShell;
    private Composite demoComposite;
    
    private Command backCommand;
    
    private Tree inheritanceTree;
    
    public String getTitle(){
        return "Tree";
    }
    
    public String getDescription(){
        return "Demonstrates the Tree component";
    }
    
    public void runDemo(Shell parent){
        demoShell = new Shell(SWT.MAX);
        demoShell.setText("org.eclipse.swt.widgets");
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));
        
        demoComposite = new Composite(demoShell, SWT.NONE);
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));
        
        
        backCommand = new Command(demoShell, Command.EXIT, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);
        demoShell.open();
        demoShell.setActive();
        createTree();
        demoShell.layout();
    }
    
    public void widgetSelected(SelectionEvent e){
        if(e.widget.equals(backCommand)){
            back();
        }
    }
    
    public void widgetDefaultSelected(SelectionEvent e){
        
    }
    
    private void back(){
        demoShell.dispose();
        demoShell = null;
    }
    
    private void createTree(){
        //create the tree
        inheritanceTree = new Tree(demoComposite, SWT.SINGLE);
        
        //the root object
        TreeItem object = new TreeItem(inheritanceTree, SWT.NONE);
        object.setText("Object");
        
        //we just add the items with their parent item as parent instead of the tree
        //this makes them show up as a child of a specific item
        TreeItem device = new TreeItem(object, SWT.NONE);
        device.setText("Device");
        
        TreeItem display = new TreeItem(device, SWT.NONE);
        display.setText("Display");
        
        TreeItem dialog = new TreeItem(object, SWT.NONE);
        dialog.setText("Dialog");
        
        TreeItem colorDialog = new TreeItem(dialog, SWT.NONE);
        colorDialog.setText("ColorDialog");
        
        TreeItem directoryDialog = new TreeItem(dialog, SWT.NONE);
        directoryDialog.setText("DirectoryDialog");
        
        TreeItem fileDialog = new TreeItem(dialog, SWT.NONE);
        fileDialog.setText("FileDialog");
        
        TreeItem fontDialog = new TreeItem(dialog, SWT.NONE);
        fontDialog.setText("FontDialog");
        
        TreeItem messageBox = new TreeItem(dialog, SWT.NONE);
        messageBox.setText("MessageBox");
        
        TreeItem event = new TreeItem(object, SWT.NONE);
        event.setText("Event");
        
        TreeItem layout = new TreeItem(object, SWT.NONE);
        layout.setText("Layout");
        
        TreeItem synchronizer = new TreeItem(object, SWT.NONE);
        synchronizer.setText("Synchronizer");
        
        TreeItem typedListener = new TreeItem(object, SWT.NONE);
        typedListener.setText("TypedListener");
        
        TreeItem widget = new TreeItem(object, SWT.NONE);
        widget.setText("Widget");
        
        TreeItem control = new TreeItem(widget, SWT.NONE);
        control.setText("Control");
                
        TreeItem button = new TreeItem(control, SWT.NONE);
        button.setText("Button");
        
        TreeItem label = new TreeItem(control, SWT.NONE);
        label.setText("Label");
        
        TreeItem progressBar = new TreeItem(control, SWT.NONE);
        progressBar.setText("ProgressBar");
        
        TreeItem scrollable = new TreeItem(control, SWT.NONE);
        scrollable.setText("Scrollable");
        
        TreeItem composite = new TreeItem(scrollable, SWT.NONE);
        composite.setText("Composite");
        
        TreeItem canvas = new TreeItem(composite, SWT.NONE);
        canvas.setText("Canvas");
        
        TreeItem decorations = new TreeItem(canvas, SWT.NONE);
        decorations.setText("Decorations");
        
        TreeItem shell = new TreeItem(decorations, SWT.NONE);
        shell.setText("Shell");
        
        TreeItem combo = new TreeItem(composite, SWT.NONE);
        combo.setText("Combo");
        
        TreeItem table = new TreeItem(composite, SWT.NONE);
        table.setText("Table");
        
        TreeItem tree = new TreeItem(composite, SWT.NONE);
        tree.setText("Tree");
        
        TreeItem list = new TreeItem(scrollable, SWT.NONE);
        list.setText("List");
        
        TreeItem text = new TreeItem(scrollable, SWT.NONE);
        text.setText("Text");
        
        TreeItem slider = new TreeItem(control, SWT.NONE);
        slider.setText("Slider");
        
        TreeItem item = new TreeItem(widget, SWT.NONE);
        item.setText("Item");
        
        TreeItem menuItem = new TreeItem(item, SWT.NONE);
        menuItem.setText("MenuItem");
        
        TreeItem tableColumn = new TreeItem(item, SWT.NONE);
        tableColumn.setText("TableColumn");
        
        TreeItem tableItem = new TreeItem(item, SWT.NONE);
        tableItem.setText("TableItem");
        
        TreeItem treeItem = new TreeItem(item, SWT.NONE);
        treeItem.setText("TreeItem");
        
        TreeItem menu = new TreeItem(widget, SWT.NONE);
        menu.setText("Menu");
        
        TreeItem scrollBar = new TreeItem(widget, SWT.NONE);
        scrollBar.setText("ScrollBar");
    }
}
