/*******************************************************************************
 * Copyright  2010 Nokia Corporation. All rights reserved.
 * Nokia and Nokia Connecting People are registered trademarks of Nokia Corporation.
 * Java and all Java-based marks are trademarks or registered trademarks of Sun Microsystems, Inc.
 * Other product and company names mentioned herein may be trademarks or trade names of their respective owners.
 *
 * Subject to the conditions below, you may, without charge:
 *
 *   Use, copy, modify and/or merge copies of this software and associated documentation files (the Software)
 *
 *   Publish, distribute, sub-license and/or sell new software derived from or incorporating the Software.
 *
 *
 *
 * This license, unmodified, shall be included with all copies or
 * substantial portions of the Software that are distributed in source code form.
 *
 * The Software cannot constitute the primary value of any new software derived from or incorporating the Software.
 *
 * Any person dealing with the Software shall not misrepresent the source of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************/
package eswtshowcase.modules;

import org.eclipse.ercp.swt.mobile.CaptionedControl;
import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.ercp.swt.mobile.ConstrainedText;
import org.eclipse.ercp.swt.mobile.DateEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
//import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * A demonstration of some of the ways of getting user input in a form.
 * Uses Text, DateEditor and ConstrainedText.
 */
public class InputDemoModule implements DemoModule, SelectionListener
{
    private Shell demoShell;
    private ScrolledComposite scrolledComposite;
    private Composite demoComposite;

    private Command backCommand;

    private Text nameBox;
    private DateEditor dateBox;
    private ConstrainedText productCodeBox;
    private Text descriptionBox;

    public String getTitle()
    {
        return "Inputs";
    }

    public String getDescription()
    {
        return "Text, DateEditor and ConstrainedText";
    }

    public void runDemo(Shell parent)
    {
        demoShell = new Shell(SWT.MAX);
        demoShell.setText("Input Demo");
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));

        scrolledComposite = new ScrolledComposite(demoShell, SWT.V_SCROLL);
        demoComposite = new Composite(scrolledComposite, SWT.NONE);
        demoComposite.setLayout(new FormLayout());
        scrolledComposite.setContent(demoComposite);

        backCommand = new Command(demoShell, Command.EXIT, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);

        //create a single line text box with a caption
        CaptionedControl nameCaption = new CaptionedControl(demoComposite, SWT.NONE);
        nameCaption.setText("Product Name");
        nameBox = new Text(nameCaption, SWT.SINGLE);

        //the form data for the name box to make it look good
        FormData nameData = new FormData();
        nameData.top = new FormAttachment(0);
        nameData.bottom = new FormAttachment(20);
        nameData.left = new FormAttachment(0,2);
        nameData.right = new FormAttachment(100,-2);
        nameCaption.setLayoutData(nameData);

        //create a caption and a date editor where
        //the user can enter both the date and the time
        CaptionedControl dateCaption = new CaptionedControl(demoComposite, SWT.NONE);
        dateCaption.setText("Date of Manufacture");
        dateBox = new DateEditor(dateCaption, SWT.NONE, DateEditor.DATE_TIME);

        //the date form data
        FormData dateData = new FormData();
        dateData.top = new FormAttachment(20);
        dateData.bottom = new FormAttachment(40);
        dateData.left = new FormAttachment(0,2);
        dateData.right = new FormAttachment(100,-2);
        dateCaption.setLayoutData(dateData);

        //we want the user to only be able to enter numbers so we create
        //a constrained text box for this
        CaptionedControl productCodeCaption = new CaptionedControl(demoComposite, SWT.NONE);
        productCodeCaption.setText("Product Code");
        productCodeBox = new ConstrainedText(productCodeCaption, SWT.NONE, ConstrainedText.NUMERIC);

        //the form data for the product code box
        FormData productCodeData = new FormData();
        productCodeData.top = new FormAttachment(40);
        productCodeData.bottom = new FormAttachment(60);
        productCodeData.left = new FormAttachment(0,2);
        productCodeData.right = new FormAttachment(100,-2);
        productCodeCaption.setLayoutData(productCodeData);

        //the big multi line box for the description, we set SWT.WRAP so
        //the text wraps automatically
        CaptionedControl descriptionCaption = new CaptionedControl(demoComposite, SWT.NONE);
        descriptionCaption.setText("Description");
        descriptionBox = new Text(descriptionCaption, SWT.MULTI|SWT.WRAP);

        //here we make the description box big enough for multiple lines to show
        FormData descriptionData = new FormData();
        descriptionData.top = new FormAttachment(60);
        descriptionData.bottom = new FormAttachment(100);
        descriptionData.left = new FormAttachment(0,2);
        descriptionData.right = new FormAttachment(100,-2);
        descriptionCaption.setLayoutData(descriptionData);

        demoComposite.setSize(demoComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
        demoComposite.layout();
        demoShell.open();
        demoShell.setActive();
    }

    public void widgetSelected(SelectionEvent e)
    {
        if(e.widget.equals(backCommand))
        {
            back();
        }
    }

    public void widgetDefaultSelected(SelectionEvent e)
    {

    }

    private void back()
    {
        demoShell.dispose();
        demoShell = null;
    }
}
