/*******************************************************************************
 * Copyright  2010 Nokia Corporation. All rights reserved.
 * Nokia and Nokia Connecting People are registered trademarks of Nokia Corporation.
 * Java and all Java-based marks are trademarks or registered trademarks of Sun Microsystems, Inc.
 * Other product and company names mentioned herein may be trademarks or trade names of their respective owners.
 *
 * Subject to the conditions below, you may, without charge:
 *
 *   Use, copy, modify and/or merge copies of this software and associated documentation files (the Software)
 *
 *   Publish, distribute, sub-license and/or sell new software derived from or incorporating the Software.
 *
 *
 *
 * This license, unmodified, shall be included with all copies or
 * substantial portions of the Software that are distributed in source code form.
 *
 * The Software cannot constitute the primary value of any new software derived from or incorporating the Software.
 *
 * Any person dealing with the Software shall not misrepresent the source of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************/
package eswtshowcase.modules;

import org.eclipse.ercp.swt.mobile.Command;
import org.eclipse.ercp.swt.mobile.QueryDialog;
import org.eclipse.ercp.swt.mobile.SortedList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

/**
 * Demonstrates the SortedList component by using a gem "database".
 */
public class SortedListDemoModule implements DemoModule, SelectionListener
{
    private Shell demoShell;
    private Shell parent;
    private Composite demoComposite;
    private SortedList demoSortedList;

    private Command backCommand;
    private Command addCommand;
    private Command removeCommand;

    private static final String[] originalContent = new String[]
    {
        "Agate", "Alexandrite", "Aquamarine", "Chrysocolla", "Chrysoprase",
        "Diamond", "Emerald", "Feldspar", "Garnet", "Hematite", "Jade",
        "Jasper", "Kunzite", "Lapis lazuli", "Malachite", "Obsidian",
        "Olivine", "Opal", "Pyrite", "Quartz", "Ruby", "Sapphire", "Spinel",
        "Sugilite", "Tanzanite", "Topaz", "Turquoise", "Tourmaline",
        "Variscite", "Zircon"
    };

    public SortedListDemoModule()
    {
    }

    public void runDemo(Shell parent)
    {
        this.parent = parent;
        demoShell = new Shell(SWT.MAX);
        demoShell.setLayout(new FillLayout(SWT.VERTICAL));
        demoShell.setText("Gem Lookup");

        demoComposite = new Composite(demoShell, SWT.NONE);
        demoComposite.setLayout(new FillLayout(SWT.VERTICAL));
        demoShell.layout();

        backCommand = new Command(demoShell, Command.EXIT, 0);
        backCommand.setText("Back");
        backCommand.addSelectionListener(this);

        addCommand = new Command(demoShell, Command.SELECT, 1);
        addCommand.setText("Add");
        addCommand.addSelectionListener(this);

        removeCommand = new Command(demoShell, Command.SELECT, 1);
        removeCommand.setText("Remove");
        removeCommand.addSelectionListener(this);

        //Create a single-selection list with a filter field and set the original content
        demoSortedList = new SortedList(demoComposite, SWT.SINGLE, SortedList.FILTER);
        demoSortedList.addSelectionListener(this);
        demoSortedList.setItems(originalContent);

        demoComposite.layout();
        demoShell.open();
        demoShell.setActive();
    }

    public String getTitle()
    {
        return "Gem Lookup";
    }

    public String getDescription()
    {
        return "Demonstrates the SortedList component";
    }

    public void widgetSelected(SelectionEvent e)
    {
        if(e.widget.equals(backCommand))
        {
            back();
        }
        else if(e.widget.equals(addCommand))
        {
            add();
        }
        else if(e.widget.equals(removeCommand))
        {
            remove();
        }
    }

    public void widgetDefaultSelected(SelectionEvent e)
    {
    }

    private void back()
    {
        demoShell.dispose();
        demoShell = null;
        parent.getShell().setActive();  // Because of Nokia 5800 bug
    }

    private void add()
    {
        //ask the user for a new gem with a QueryDialog
        QueryDialog dialog = new QueryDialog(demoShell, QueryDialog.STANDARD);
        dialog.setPromptText("Add a Gem", "");
        String itemToAdd = dialog.open();

        //if the user pressed the Cancel button
        if(itemToAdd == null)
        {
            return;
        }

        //add the result into the list
        demoSortedList.add(itemToAdd);
    }

    private void remove()
    {
        //because it's a single selection list the focused item is the selected item
        String selection = demoSortedList.getFocus();

        //display a confirmation box with the Yes and No buttons
        MessageBox confirm = new MessageBox(demoShell,
                                            SWT.ICON_QUESTION | SWT.YES | SWT.NO);
        confirm.setMessage("Are you sure you want to remove " + selection +
                           " from the lookup?");
        int result = confirm.open();
        //remove the item if the user pressed Yes
        if(result == SWT.YES)
        {
            demoSortedList.remove(selection);
        }
        else
        {
            return;
        }
    }
}